"""
Test models
"""

# Django
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.test import TestCase, modify_settings

# AA Discord Announcements
from aa_discord_announcements.models import PingTarget, Webhook


class TestModels(TestCase):
    """
    Test our models
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

    def test_discord_webhook_invalid_webhook_url_should_throw_exception(self):
        """
        Test if we get a ValidationError for a Discord webhook
        :return:
        """

        # given
        webhook = Webhook(
            url=(
                "https://discord.com/api/webhooks/754119343402302920F/x-BfFCdEG"
                "-qGg_39_mFUqRSLoz2dm6Oa8vxNdaAxZdgKOAyesVpy-Bzf8wDU_vHdFpm-"
            ),
        )

        # when
        with self.assertRaises(ValidationError):
            webhook.clean()

        with self.assertRaisesMessage(
            ValidationError,
            expected_message=(
                "Invalid webhook URL. The webhook URL you entered does not match any "
                "known format for a Discord webhook. Please check the "
                "webhook URL."
            ),
        ):
            webhook.clean()

    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.services.modules.discord"})
    def test_should_raise_validation_error_for_not_activated_discord_service(self):
        """
        Test should raise a validation error when Discord service is not active
        :return:
        """

        # given
        announcement_target = PingTarget(name=self.group, discord_id=123456789)

        # when
        with self.assertRaises(ValidationError):
            announcement_target.clean()

        with self.assertRaisesMessage(
            ValidationError,
            expected_message="You might want to install the Discord service first …",
        ):
            announcement_target.clean()

    def test_should_raise_validation_error_on_save(self):
        """
        Test should raise validation error on save
        :return:
        :rtype:
        """

        ping_target = PingTarget(name=self.group)

        with self.assertRaises(ValidationError):
            ping_target.save()

    def test_should_return_ping_target_model_string_name(self):
        """
        Test should return the PingTarget model string name
        :return:
        :rtype:
        """

        ping_target = PingTarget(name=self.group)

        self.assertEqual(str(ping_target), self.group.name)

    def test_should_return_webhook_model_string_name(self):
        """
        Test should return the Webhook model string name
        :return:
        :rtype:
        """

        webhook = Webhook(
            name="Test Webhook",
            url=(
                "https://discord.com/api/webhooks/754119343402302920F/x-BfFCdEG"
                "-qGg_39_mFUqRSLoz2dm6Oa8vxNdaAxZdgKOAyesVpy-Bzf8wDU_vHdFpm-"
            ),
        )

        webhook.save()

        self.assertEqual(str(webhook), "Test Webhook")



"""
Test ajax calls
"""

# Standard Library
from http import HTTPStatus

# Django
from django.contrib.auth.models import Group
from django.test import TestCase
from django.urls import reverse

# AA Discord Announcements
from aa_discord_announcements.tests.utils import create_fake_user


class TestAccess(TestCase):
    """
    Test access to ajax calls
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

        # User cannot access aa_discord_announcements
        cls.user_1001 = create_fake_user(1001, "Peter Parker")

        # User can access aa_discord_announcements
        cls.user_1002 = create_fake_user(
            1002, "Bruce Wayne", permissions=["aa_discord_announcements.basic_access"]
        )

    def test_ajax_get_announcement_targets_no_access(self):
        """
        Test ajax call to get announcement targets available for
        the current user without access to it
        :return:
        """

        # given
        self.client.force_login(self.user_1001)

        # when
        res = self.client.get(
            reverse("aa_discord_announcements:ajax_get_announcement_targets")
        )

        # then
        self.assertEqual(res.status_code, HTTPStatus.FOUND)

    def test_ajax_get_announcement_targets_general(self):
        """
        Test ajax call to get announcement targets available for the current user
        :return:
        """

        # given
        self.client.force_login(self.user_1002)

        # when
        res = self.client.get(
            reverse("aa_discord_announcements:ajax_get_announcement_targets")
        )

        # then
        self.assertEqual(res.status_code, HTTPStatus.OK)

    def test_ajax_get_webhooks_no_access(self):
        """
        Test ajax call to get webhooks available for
        the current user without access to it
        :return:
        """

        # given
        self.client.force_login(self.user_1001)

        # when
        res = self.client.get(reverse("aa_discord_announcements:ajax_get_webhooks"))

        # then
        self.assertEqual(res.status_code, HTTPStatus.FOUND)

    def test_ajax_get_webhooks_general(self):
        """
        Test ajax call to get webhooks available for the current user
        :return:
        """

        # given
        self.client.force_login(self.user_1002)

        # when
        res = self.client.get(reverse("aa_discord_announcements:ajax_get_webhooks"))

        # then
        self.assertEqual(res.status_code, HTTPStatus.OK)

    def test_ajax_create_announcement_no_access(self):
        """
        Test ajax call to create an announcement is not available for
        a user without access to it
        :return:
        """

        # given
        self.client.force_login(self.user_1001)

        # when
        res = self.client.get(
            reverse("aa_discord_announcements:ajax_create_announcement")
        )

        # then
        self.assertEqual(res.status_code, HTTPStatus.FOUND)

    def test_ajax_create_announcement_general(self):
        """
        Test ajax call to create an announcement is available for the current user
        :return:
        """

        # given
        self.client.force_login(self.user_1002)

        # when
        res = self.client.get(
            reverse("aa_discord_announcements:ajax_create_announcement")
        )

        # then
        self.assertEqual(res.status_code, HTTPStatus.OK)

    def test_ajax_create_announcement_with_form_data(self):
        """
        Test ajax call to create an announcement for the current user with form data
        :return:
        """

        # given
        self.client.force_login(self.user_1002)
        form_data = {
            "announcement_target": "@here",
            "announcement_channel": "",
            "announcement_text": "Borg to slaughter!",
        }

        # when
        response = self.client.post(
            reverse("aa_discord_announcements:ajax_create_announcement"), data=form_data
        )

        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertTemplateUsed(
            response,
            "aa_discord_announcements/partials/announcement/copy-paste-text.html",
        )
        self.assertContains(response, "@here")
        self.assertContains(response, "Borg to slaughter!")



"""
A couple of variables to use throughout the app
"""

# Standard Library
from importlib import metadata

# Django
from django.utils.translation import gettext_lazy as _

__version__ = metadata.version("aa-discord-announcements")
__title__ = _("Discord Announcements")

del metadata



"""
Tests for the template tags
"""

# Django
from django.template import Context, Template
from django.test import TestCase

# AA Discord Announcements
from aa_discord_announcements import __version__


class TestVersionedStatic(TestCase):
    """
    Test aa_discord_announcements_versioned_static
    """

    def test_versioned_static(self):
        """
        Test versioned static template tag
        :return:
        """

        context = Context({"version": __version__})
        template_to_render = Template(
            "{% load aa_discord_announcements_versioned_static %}"
            "{% aa_discord_announcements_static 'aa_discord_announcements/css/aa-discord-announcements.min.css' %}"  # pylint: disable=line-too-long
        )

        rendered_template = template_to_render.render(context)

        self.assertInHTML(
            f'/static/aa_discord_announcements/css/aa-discord-announcements.min.css?v={context["version"]}',  # pylint: disable=line-too-long
            rendered_template,
        )
